package me.totalfreedom.totalfreedommod.command;

import me.totalfreedom.totalfreedommod.admin.Admin;
import me.totalfreedom.totalfreedommod.config.ConfigEntry;
import me.totalfreedom.totalfreedommod.player.PlayerData;
import me.totalfreedom.totalfreedommod.rank.Rank;
import me.totalfreedom.totalfreedommod.util.FConverter;
import me.totalfreedom.totalfreedommod.util.FUtil;
import net.kyori.adventure.text.format.TextColor;
import net.kyori.adventure.text.minimessage.tag.Tag;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import net.kyori.adventure.text.minimessage.tag.resolver.TagResolver;
import org.apache.commons.lang.StringUtils;
import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

@CommandPermissions(rank = Rank.ADMIN, source = SourceType.ONLY_IN_GAME)
@CommandParameters(description = "Manage your admin entry.", usage = "/<command> [-o <admin name>] <clearips | clearip <ip> | convertacformat | setacformat <format> | clearacformat> | syncroles>")
public class Command_myadmin extends FreedomCommand
{
    @Override
    public boolean run(CommandSender sender, Player playerSender, Command cmd, String commandLabel, String[] args, boolean senderIsConsole)
    {
        if (args.length < 1)
        {
            return false;
        }

        Player init = null;
        Admin target = getAdmin(playerSender);
        Player targetPlayer = playerSender;

        // -o switch
        if (args[0].equals("-o"))
        {
            checkRank(Rank.SENIOR_ADMIN);
            init = playerSender;
            targetPlayer = getPlayer(args[1]);
            if (targetPlayer == null)
            {
                msg(PLAYER_NOT_FOUND);
                return true;
            }

            target = getAdmin(targetPlayer);
            if (target == null)
            {
                msgNew("<red>That player is not an admin");
                return true;
            }

            // Shift 2
            args = Arrays.copyOfRange(args, 2, args.length);
            if (args.length < 1)
            {
                return false;
            }
        }

        final String targetIp = FUtil.getIp(targetPlayer);

        switch (args[0])
        {
            case "clearips":
            {
                if (args.length != 1)
                {
                    return false; // Double check: the player might mean "clearip"
                }

                if (init == null)
                {
                    FUtil.adminAction(sender.getName(), "Clearing my IPs", true);
                } else
                {
                    FUtil.adminAction(sender.getName(), "Clearing " + target.getName() + "'s IPs", true);
                }

                int counter = target.getIps().size() - 1;
                target.clearIPs();
                target.addIp(targetIp);

                plugin.al.save(target);
                plugin.al.updateTables();
                plugin.pl.syncIps(target);

                msgNew("<count> IPs removed.", Placeholder.unparsed("count", String.valueOf(counter)));
                msgNew(targetPlayer, "<ip> is now your only IP address.", Placeholder.unparsed("ip", String.valueOf(target.getIps().get(0))));
                return true;
            }

            case "clearip":
            {
                if (args.length != 2)
                {
                    return false; // Double check: the player might mean "clearips"
                }

                if (!target.getIps().contains(args[1]))
                {
                    if (init == null)
                    {
                        msgNew("<red>That IP is not registered to you.");
                    }
                    else
                    {
                        msgNew("<red>That IP does not belong to that player.");
                    }
                    return true;
                }

                if (targetIp.equals(args[1]))
                {
                    if (init == null)
                    {
                        msgNew("<red>You cannot remove your current IP.");
                    }
                    else
                    {
                        msgNew("<red>You cannot remove that admin's current IP.");
                    }
                    return true;
                }

                FUtil.adminAction(sender.getName(), "Removing an IP" + (init == null ? "" : " from " + targetPlayer.getName() + "'s IPs"), true);

                target.removeIp(args[1]);
                plugin.al.save(target);
                plugin.al.updateTables();

                plugin.pl.syncIps(target);

                msgNew("Removed the IP <ip>.", Placeholder.unparsed("ip", args[1]));
                msgNew("Current IPs: <ips>", Placeholder.unparsed("ip", StringUtils.join(target.getIps(), ", ")));
                return true;
            }

            case "convertacformat":
            {
                if (!FConverter.needsConversion(target.getAcFormat()))
                {
                    msgNew("<red>That format does not need to be converted.");
                }
                else
                {
                    msgNew("<yellow>Converting format...");
                    target.setAcFormat(FConverter.convertAdminChatFormat(target.getAcFormat()));
                    plugin.al.save(target);
                    plugin.al.updateTables();
                    msgNew("<green>Format conversion complete.");
                }

                return true;
            }

            case "setacformat":
            case "setscformat":
            {
                String format = StringUtils.join(args, " ", 1, args.length);
                if (FConverter.needsConversion(format))
                {
                    msgNew("<yellow>⚠ Admin chat formats use MiniMessage now. We'll convert your format for you, but consider using MiniMessage next time: https://docs.advntr.dev/minimessage/format.html ⚠");
                    format = FConverter.convertAdminChatFormat(format);
                }

                target.setAcFormat(format);
                plugin.al.save(target);
                plugin.al.updateTables();
                msgNew("Set admin chat format to \"<format>\"", Placeholder.unparsed("format", format));
                msgNew("Example:");
                msgNew(format, Placeholder.unparsed("name", "ExampleAdmin"),
                        Placeholder.unparsed("rank", Rank.ADMIN.getAbbr()),
                        TagResolver.resolver("rankcolor", Tag.styling(lol -> lol.color(TextColor.color(Rank.ADMIN.getColor().getColor().getRGB())))),
                        Placeholder.unparsed("message", "The quick brown fox jumped over the lazy dog."));
                return true;
            }

            case "clearacformat":
            case "clearscformat":
            {
                target.setAcFormat(null);
                plugin.al.save(target);
                plugin.al.updateTables();
                msgNew("Cleared admin chat format.");
                return true;
            }

            case "syncroles":
            {
                if (plugin.dc != null && plugin.dc.isEnabled())
                {
                    if (!ConfigEntry.DISCORD_ROLE_SYNC.getBoolean())
                    {
                        msgNew("<red>Role syncing is not enabled.");
                        return true;
                    }
                    PlayerData playerData = plugin.pl.getData(target.getName());
                    if (playerData.getDiscordID() == null)
                    {
                        msgNew("<red>Please run /linkdiscord first!");
                        return true;
                    }
                    boolean synced = plugin.dc.syncRoles(target, playerData.getDiscordID());
                    if (synced)
                    {
                        msgNew("<green>Successfully synced your roles.");
                    }
                    else
                    {
                        msgNew("<red>Failed to sync your roles, please check the console.");
                    }
                }

                return true;
            }

            default:
            {
                return false;
            }
        }
    }

    @Override
    public List<String> getTabCompleteOptions(CommandSender sender, Command command, String alias, String[] args)
    {
        if (!plugin.al.isAdmin(sender))
        {
            return Collections.emptyList();
        }

        List<String> singleArguments = Arrays.asList("clearips", "setscformat", "setacformat");
        List<String> doubleArguments = Arrays.asList("clearip", "clearscformat", "clearacformat", "syncroles");
        if (args.length == 1)
        {
            List<String> options = new ArrayList<>();
            options.add("-o");
            options.addAll(singleArguments);
            options.addAll(doubleArguments);
            return options;
        } else if (args.length == 2)
        {
            if (args[0].equalsIgnoreCase("-o"))
            {
                return FUtil.getPlayerList();
            } else
            {
                if (doubleArguments.contains(args[0]) && args[0].equalsIgnoreCase("clearip"))
                {
                    List<String> ips = plugin.al.getAdmin(sender).getIps();
                    ips.remove(FUtil.getIp((Player) sender));
                    return ips;
                }
            }
        } else if (args.length == 3 && args[0].equalsIgnoreCase("-o"))
        {
            List<String> options = new ArrayList<>();
            options.addAll(singleArguments);
            options.addAll(doubleArguments);
            return options;
        } else if (args.length == 4 && args[0].equalsIgnoreCase("-o") && args[2].equalsIgnoreCase("clearip"))
        {
            Admin admin = plugin.al.getEntryByName(args[1]);
            if (admin != null)
            {
                return admin.getIps();
            }
        }
        return FUtil.getPlayerList();
    }
}