package me.totalfreedom.totalfreedommod.command;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import me.totalfreedom.totalfreedommod.player.PlayerData;
import me.totalfreedom.totalfreedommod.rank.Rank;
import me.totalfreedom.totalfreedommod.util.FUtil;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

@CommandPermissions(rank = Rank.ADMIN, source = SourceType.BOTH)
@CommandParameters(description = "Manage notes for a player", usage = "/<command> <name> <list | add <note> | remove <id> | clear>")
public class Command_notes extends FreedomCommand
{
    @Override
    public boolean run(CommandSender sender, Player playerSender, Command cmd, String commandLabel, String[] args, boolean senderIsConsole)
    {
        if (args.length < 2)
        {
            return false;
        }

        PlayerData playerData;

        final Player player = getPlayer(args[0]);
        if (player == null)
        {
            final PlayerData entry = plugin.pl.getData(args[0]);

            if (entry == null)
            {
                msg(PLAYER_NOT_FOUND);
                return true;
            }

            playerData = plugin.pl.getData(entry.getName());
        }
        else
        {
            playerData = plugin.pl.getData(player);
        }

        switch (args[1])
        {
            case "list":
            {
                msgNew("<green>Player notes for <player>:", Placeholder.unparsed("player", playerData.getName()));
                int id = 1;
                for (String note : playerData.getNotes())
                {
                    msgNew("<num>. <note>",
                            Placeholder.unparsed("num", String.valueOf(id)),
                            Placeholder.unparsed("note", note));
                    id++;
                }
                return true;
            }

            case "add":
            {
                if (args.length < 3)
                {
                    return false;
                }
                String note = sender.getName() + ": " + StringUtils.join(ArrayUtils.subarray(args, 2, args.length), " ");
                playerData.addNote(note);
                plugin.pl.save(playerData);
                msgNew("<green>Note added.");
                return true;
            }

            case "remove":
            {
                if (args.length < 3)
                {
                    return false;
                }

                int id;
                try
                {
                    id = Integer.parseInt(args[2]);
                }
                catch (NumberFormatException e)
                {
                    msgNew("<red>Invalid number: <num>", Placeholder.unparsed("num", args[2]));
                    return true;
                }

                id--;

                if (playerData.removeNote(id))
                {
                    plugin.pl.save(playerData);
                    msgNew("<green>Note removed.");
                }
                else
                {
                    msgNew("<red>No note with the ID of <id> exists.", Placeholder.unparsed("id", args[2]));
                }

                return true;
            }

            case "clear":
            {
                int count = playerData.getNotes().size();
                playerData.clearNotes();
                plugin.pl.save(playerData);
                msgNew("<green>Cleared <count> notes.", Placeholder.unparsed("count", String.valueOf(count)));
                return true;
            }

            default:
            {
                return false;
            }
        }
    }

    @Override
    public List<String> getTabCompleteOptions(CommandSender sender, Command command, String alias, String[] args)
    {
        if (args.length == 1)
        {
            return FUtil.getPlayerList();
        }
        else if (args.length == 2)
        {
            return Arrays.asList("list", "add", "remove", "clear");
        }
        else if (args.length > 2 && (args[1].equals("add")))
        {
            return FUtil.getPlayerList();
        }
        return Collections.emptyList();
    }
}