package me.totalfreedom.totalfreedommod.command;

import java.util.Objects;
import me.totalfreedom.totalfreedommod.config.ConfigEntry;
import me.totalfreedom.totalfreedommod.player.FPlayer;
import me.totalfreedom.totalfreedommod.rank.Rank;
import me.totalfreedom.totalfreedommod.util.FUtil;
import me.totalfreedom.totalfreedommod.util.Groups;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import org.bukkit.Material;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;

@CommandPermissions(rank = Rank.OP, source = SourceType.ONLY_IN_GAME)
@CommandParameters(description = "Throw a mob in the direction you are facing when you right click with a bone.",
        usage = "/<command> <mobtype [speed] | off | list>")
public class Command_tossmob extends FreedomCommand
{
    @Override
    public boolean run(CommandSender sender, Player playerSender, Command cmd, String commandLabel, String[] args, boolean senderIsConsole)
    {
        if (!ConfigEntry.TOSSMOB_ENABLED.getBoolean())
        {
            msgNew("<red>Tossmob is currently disabled.");
            return true;
        }

        if (args.length == 0)
        {
            return false;
        }

        FPlayer playerData = plugin.pl.getPlayer(playerSender);

        EntityType type = null;
        if (args[0].equalsIgnoreCase("off"))
        {
            playerData.disableMobThrower();
            msgNew("<green>Tossmob has been turned off.");
            return true;
        }

        if (args[0].equalsIgnoreCase("list"))
        {
            msgNew("<green>Supported mobs: <mobs>", Placeholder.unparsed("mobs", String.join(", ", Groups.MOB_TYPES.stream().map(Enum::name).toList())));
            return true;
        }

        for (EntityType loop : EntityType.values())
        {
            if (loop != null && loop.name().equalsIgnoreCase(args[0]))
            {
                type = loop;
                break;
            }
        }

        if (type == null)
        {
            msgNew("<red>Unknown entity type: <type>", Placeholder.unparsed("type", args[0]));
            return true;
        }

        if (!Groups.MOB_TYPES.contains(type))
        {
            msgNew("<red><entity> is an entity, however it is not a mob.", Placeholder.unparsed("entity", FUtil.formatName(type.name())));
            return true;
        }

        double speed = 1.0;
        if (args.length >= 2)
        {
            try
            {
                speed = Double.parseDouble(args[1]);
            }
            catch (NumberFormatException ex)
            {
                msgNew("<red>The input provided is not a valid integer.");
                return true;
            }
        }

        if (speed < 1.0)
        {
            speed = 1.0;
        }
        else if (speed > 5.0)
        {
            speed = 5.0;
        }

        playerData.enableMobThrower(type, speed);
        msgNew("<green>Tossmob is enabled. Mob: <type> - Speed: <speed>.",
                Placeholder.unparsed("type", String.valueOf(type)),
                Placeholder.unparsed("speed", String.valueOf(speed)));
        msgNew("<green>Right click while holding a Bone to throw mobs!");
        msgNew("<green>Type '/tossmob off' to disable. - By Madgeek1450");

        Objects.requireNonNull(playerSender.getEquipment()).setItemInMainHand(new ItemStack(Material.BONE, 1));
        return true;
    }
}