package me.totalfreedom.totalfreedommod.command;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import me.totalfreedom.totalfreedommod.rank.Rank;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import org.apache.commons.lang.StringUtils;
import org.bukkit.Material;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.HumanEntity;
import org.bukkit.entity.Player;

@CommandPermissions(rank = Rank.OP, source = SourceType.BOTH)
@CommandParameters(description = "See who has an item and optionally clear the specified item.", usage = "/<command> <item> [clear]", aliases = "wh")
public class Command_whohas extends FreedomCommand
{
    @Override
    public boolean run(CommandSender sender, Player playerSender, Command cmd, String commandLabel, String[] args, boolean senderIsConsole)
    {
        if (args.length < 1)
        {
            return false;
        }

        final boolean doClear = args.length >= 2 && "clear".equalsIgnoreCase(args[1]);

        final String materialName = args[0];
        Material material = Material.matchMaterial(materialName);

        if (material == null)
        {
            msgNew("<red>Invalid item: <type>", Placeholder.unparsed("type", materialName));
            return true;
        }

        List<? extends Player> players = server.getOnlinePlayers().stream().filter(player -> player.getInventory().contains(material) &&
                (!plugin.al.isVanished(player.getUniqueId()) || plugin.al.isAdmin(sender))).toList();

        if (plugin.al.isAdmin(sender) && doClear)
        {
            players.stream().filter(player -> !plugin.al.isAdmin(player)).forEach(player ->
                    player.getInventory().remove(material));
        }

        if (players.isEmpty())
        {
            msgNew("There are no players with that item.");
        }
        else
        {
            msgNew("Players with item <type>: <players>", Placeholder.unparsed("type", material.name()),
                    Placeholder.unparsed("players", StringUtils.join(players.stream().map(HumanEntity::getName).toList(), ", ")));
        }

        return true;
    }

    @Override
    public List<String> getTabCompleteOptions(CommandSender sender, Command command, String alias, String[] args)
    {
        if (args.length == 1)
        {
            return Arrays.stream(Material.values()).map(Enum::name).toList();
        }

        if (args.length == 2 && plugin.al.isAdmin(sender))
        {
            return Collections.singletonList("clear");
        }

        return Collections.emptyList();
    }
}