package me.totalfreedom.discord;

import discord4j.core.DiscordClientBuilder;
import discord4j.core.GatewayDiscordClient;
import discord4j.core.event.domain.interaction.ChatInputInteractionEvent;
import discord4j.core.object.entity.Guild;
import me.totalfreedom.discord.command.HelpCommand;
import me.totalfreedom.discord.command.ListCommand;
import me.totalfreedom.discord.command.TPSCommand;
import me.totalfreedom.discord.handling.CommandHandler;
import me.totalfreedom.discord.react.ReactiveBukkitScheduler;
import me.totalfreedom.discord.util.SnowflakeEntry;
import me.totalfreedom.discord.util.TFM_Bridge;
import me.totalfreedom.totalfreedommod.config.ConfigEntry;
import me.totalfreedom.totalfreedommod.player.PlayerData;
import me.totalfreedom.totalfreedommod.util.FLog;
import org.bukkit.Bukkit;
import reactor.core.publisher.Mono;

import java.util.HashMap;
import java.util.Map;

public class Bot
{
    private final GatewayDiscordClient client;
    private final TFM_Bridge tfm;
    private final HashMap<String, PlayerData> LINK_CODES = new HashMap<>();
    public Bot()
    {
        //Creates the gateway client and connects to the gateway
        this.client = DiscordClientBuilder.create(ConfigEntry.DISCORD_TOKEN.getString())
                .build()
                .login()
                .block();

        this.tfm = new TFM_Bridge(this);
    }

    public void initialize() {
        if (client == null) throw new IllegalStateException();
    }

    public String formatBotTag() {
        return client.getSelf()
                .blockOptional()
                .orElseThrow()
                .getUsername() + "#" + client.getSelf()
                .blockOptional()
                .orElseThrow()
                .getDiscriminator();
    }

    public TFM_Bridge getTFM()
    {
        return tfm;
    }

    public Mono<Guild> getGuildById()
    {
        return client.getGuildById(SnowflakeEntry.SERVER_ID.getSnowflake());
    }

    public GatewayDiscordClient getClient()
    {
        return client;
    }

    public Map<String, PlayerData> getLinkCodes()
    {
        return LINK_CODES;
    }

    public boolean shouldISendReport()
    {
        if (ConfigEntry.DISCORD_REPORT_CHANNEL_ID.getString().isEmpty())
        {
            return false;
        }

        if (ConfigEntry.DISCORD_SERVER_ID.getString().isEmpty())
        {
            FLog.severe("No Discord server ID was specified in the config, but there is a report channel ID.");
            return false;
        }

        Guild server = client.getGuildById(SnowflakeEntry.SERVER_ID.getSnowflake()).block();
        if (server == null)
        {
            FLog.severe("The Discord server ID specified is invalid, or the bot is not on the server.");
            return false;
        }

        return true;
    }
}
